@@ -13,8 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""
-Token-based Authentication Middleware
+r"""
+Token-based Authentication Middleware.
 
 This WSGI component:
 
@@ -299,7 +299,8 @@ _OPTS = [
                help='Determines the frequency at which the list of revoked'
                ' tokens is retrieved from the Identity service (in seconds). A'
                ' high number of revocation events combined with a low cache'
-               ' duration may significantly reduce performance.'),
+               ' duration may significantly reduce performance. Only valid'
+               ' for PKI tokens.'),
     cfg.StrOpt('memcache_security_strategy',
                default='None',
                choices=('None', 'MAC', 'ENCRYPT'),
@@ -376,6 +377,7 @@ CONF = cfg.CONF
 CONF.register_opts(_OPTS, group=_base.AUTHTOKEN_GROUP)
 
 _LOG = logging.getLogger(__name__)
+_CACHE_INVALID_INDICATOR = 'invalid'
 
 
 class _BIND_MODE(object):
@@ -414,8 +416,8 @@ def _conf_values_type_convert(conf):
             if v is not None:
                 type_, dest = opt_types[k]
                 v = type_(v)
-        except KeyError:
-            # This option is not known to auth_token.
+        except KeyError:  # nosec
+            # This option is not known to auth_token. v is not converted.
             pass
         except ValueError as e:
             raise ksm_exceptions.ConfigurationError(
@@ -426,7 +428,20 @@ def _conf_values_type_convert(conf):
 
 
 def _get_project_version(project):
-    return pkg_resources.get_distribution(project).version
+    try:
+        return pkg_resources.get_distribution(project).version
+    except pkg_resources.DistributionNotFound:
+        return "unknown"
+
+
+def _uncompress_pkiz(token):
+    # TypeError If the signed_text is not zlib compressed binascii.Error if
+    # signed_text has incorrect base64 padding (py34)
+
+    try:
+        return cms.pkiz_uncompress(token)
+    except (TypeError, binascii.Error):
+        raise ksm_exceptions.InvalidToken(token)
 
 
 class BaseAuthProtocol(object):
@@ -481,7 +496,8 @@ class BaseAuthProtocol(object):
             try:
                 data, user_auth_ref = self._do_fetch_token(request.user_token)
                 self._validate_token(user_auth_ref)
-                self._confirm_token_bind(user_auth_ref, request)
+                if not request.service_token:
+                    self._confirm_token_bind(user_auth_ref, request)
             except ksm_exceptions.InvalidToken:
                 self.log.info(_LI('Invalid user token'))
                 request.user_token_valid = False
@@ -517,7 +533,7 @@ class BaseAuthProtocol(object):
             raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
 
     def _do_fetch_token(self, token):
-        """Helper method to fetch a token and convert it into an AccessInfo"""
+        """Helper method to fetch a token and convert it into an AccessInfo."""
         data = self.fetch_token(token)
 
         try:
@@ -629,25 +645,26 @@ class AuthProtocol(BaseAuthProtocol):
         # conf value into correct type.
         self._conf = _conf_values_type_convert(conf)
 
-        # NOTE(sileht): If we don't want to use oslo.config global object
-        # we can set the paste "oslo_config_project" and the middleware
-        # will load the configuration with a local oslo.config object.
-        self._local_oslo_config = None
-        if 'oslo_config_project' in conf:
+        # NOTE(sileht, cdent): If we don't want to use oslo.config global
+        # object there are two options: set "oslo_config_project" in
+        # paste.ini and the middleware will load the configuration with a
+        # local oslo.config object or the caller which instantiates
+        # AuthProtocol can pass in an existing oslo.config as the
+        # value of the "oslo_config_config" key in conf. If both are
+        # set "olso_config_config" is used.
+        self._local_oslo_config = conf.get('oslo_config_config')
+        if (not self._local_oslo_config) and ('oslo_config_project' in conf):
             if 'oslo_config_file' in conf:
                 default_config_files = [conf['oslo_config_file']]
             else:
                 default_config_files = None
-
-            # For unit tests, support passing in a ConfigOpts in
-            # oslo_config_config.
-            self._local_oslo_config = conf.get('oslo_config_config',
-                                               cfg.ConfigOpts())
+            self._local_oslo_config = cfg.ConfigOpts()
             self._local_oslo_config(
-                {}, project=conf['oslo_config_project'],
+                [], project=conf['oslo_config_project'],
                 default_config_files=default_config_files,
                 validate_default_values=True)
 
+        if self._local_oslo_config:
             self._local_oslo_config.register_opts(_OPTS,
                                                   group=_base.AUTHTOKEN_GROUP)
             self._local_oslo_config.register_opts(_auth.OPTS,
@@ -739,7 +756,16 @@ class AuthProtocol(BaseAuthProtocol):
                 self.log.info(_LI('Deferring reject downstream'))
             else:
                 self.log.info(_LI('Rejecting request'))
-                self._reject_request()
+                message = 'The request you have made requires authentication.'
+                body = {'error': {
+                    'code': 401,
+                    'title': 'Unauthorized',
+                    'message': message,
+                }}
+                raise webob.exc.HTTPUnauthorized(
+                    body=jsonutils.dumps(body),
+                    headers=self._reject_auth_headers,
+                    content_type='application/json')
 
         if request.user_token_valid:
             user_auth_ref = request.token_auth._user_auth_ref
@@ -772,17 +798,6 @@ class AuthProtocol(BaseAuthProtocol):
         header_val = 'Keystone uri=\'%s\'' % self._auth_uri
         return [('WWW-Authenticate', header_val)]
 
-    def _reject_request(self):
-        """Redirect client to auth server.
-
-        :param env: wsgi request environment
-        :param start_response: wsgi response callback
-        :returns: HTTPUnauthorized http response
-
-        """
-        raise webob.exc.HTTPUnauthorized(body='Authentication required',
-                                         headers=self._reject_auth_headers)
-
     def _token_hashes(self, token):
         """Generate a list of hashes that the current token may be cached as.
 
@@ -811,7 +826,6 @@ class AuthProtocol(BaseAuthProtocol):
 
         :returns: token data if found else None.
         """
-
         for token in token_hashes:
             cached = self._token_cache.get(token)
 
@@ -833,18 +847,29 @@ class AuthProtocol(BaseAuthProtocol):
             cached = self._cache_get_hashes(token_hashes)
 
             if cached:
-                data = cached
+                if cached == _CACHE_INVALID_INDICATOR:
+                    self._LOG.debug('Cached token is marked unauthorized')
+                    raise ksm_exceptions.InvalidToken()
 
                 if self._check_revocations_for_cached:
                     # A token might have been revoked, regardless of initial
                     # mechanism used to validate it, and needs to be checked.
                     self._revocations.check(token_hashes)
+
+                # NOTE(jamielennox): Cached values used to be stored as a tuple
+                # of data and expiry time. They no longer are but we have to
+                # allow some time to transition the old format so if it's a
+                # tuple just use the data.
+                if len(cached) == 2:
+                    cached = cached[0]
+
+                data = cached
             else:
                 data = self._validate_offline(token, token_hashes)
                 if not data:
                     data = self._identity_server.verify_token(token)
 
-                self._token_cache.store(token_hashes[0], data)
+                self._token_cache.set(token_hashes[0], data)
 
         except (ksa_exceptions.ConnectFailure,
                 ksa_exceptions.RequestTimeout,
@@ -855,24 +880,27 @@ class AuthProtocol(BaseAuthProtocol):
         except ksm_exceptions.InvalidToken:
             self.log.debug('Token validation failure.', exc_info=True)
             if token_hashes:
-                self._token_cache.store_invalid(token_hashes[0])
+                self._token_cache.set(token_hashes[0],
+                                      _CACHE_INVALID_INDICATOR)
             self.log.warning(_LW('Authorization failed for token'))
             raise
-        except Exception:
-            self.log.critical(_LC('Unable to validate token'), exc_info=True)
-            raise webob.exc.HTTPInternalServerError()
 
         return data
 
     def _validate_offline(self, token, token_hashes):
+        if cms.is_pkiz(token):
+            token_data = _uncompress_pkiz(token)
+            inform = cms.PKIZ_CMS_FORM
+        elif cms.is_asn1_token(token):
+            token_data = cms.token_to_cms(token)
+            inform = cms.PKI_ASN1_FORM
+        else:
+            # Can't do offline validation for this type of token.
+            return
+
         try:
-            if cms.is_pkiz(token):
-                verified = self._verify_pkiz_token(token, token_hashes)
-            elif cms.is_asn1_token(token):
-                verified = self._verify_signed_token(token, token_hashes)
-            else:
-                # Can't do offline validation for this type of token.
-                return
+            self._revocations.check(token_hashes)
+            verified = self._cms_verify(token_data, inform)
         except ksc_exceptions.CertificateConfigError:
             self.log.warning(_LW('Fetch certificate config failed, '
                                  'fallback to online validation.'))
@@ -903,7 +931,7 @@ class AuthProtocol(BaseAuthProtocol):
             raise ksm_exceptions.InvalidToken(msg)
 
     def _cms_verify(self, data, inform=cms.PKI_ASN1_FORM):
-        """Verifies the signature of the provided data's IAW CMS syntax.
+        """Verify the signature of the provided data's IAW CMS syntax.
 
         If either of the certificate files might be missing, fetch them and
         retry.
@@ -939,24 +967,6 @@ class AuthProtocol(BaseAuthProtocol):
                 self.log.error(_LE('CMS Verify output: %s'), err.output)
                 raise
 
-    def _verify_signed_token(self, signed_text, token_ids):
-        """Check that the token is unrevoked and has a valid signature."""
-        self._revocations.check(token_ids)
-        formatted = cms.token_to_cms(signed_text)
-        verified = self._cms_verify(formatted)
-        return verified
-
-    def _verify_pkiz_token(self, signed_text, token_ids):
-        self._revocations.check(token_ids)
-        try:
-            uncompressed = cms.pkiz_uncompress(signed_text)
-            verified = self._cms_verify(uncompressed, inform=cms.PKIZ_CMS_FORM)
-            return verified
-        # TypeError If the signed_text is not zlib compressed
-        # binascii.Error if signed_text has incorrect base64 padding (py34)
-        except (TypeError, binascii.Error):
-            raise ksm_exceptions.InvalidToken(signed_text)
-
     def _fetch_signing_cert(self):
         self._signing_directory.write_file(
             self._SIGNING_CERT_FILE_NAME,
@@ -968,18 +978,10 @@ class AuthProtocol(BaseAuthProtocol):
             self._identity_server.fetch_ca_cert())
 
     def _get_auth_plugin(self):
-        # NOTE(jamielennox): Ideally this would use get_from_conf_options
+        # NOTE(jamielennox): Ideally this would use load_from_conf_options
         # however that is not possible because we have to support the override
-        # pattern we use in _conf_get. There is a somewhat replacement for this
-        # in keystoneclient in load_from_options_getter which should be used
-        # when available. Until then this is essentially a copy and paste of
-        # the ksc load_from_conf_options code because we need to get a fix out
-        # for this quickly.
-
-        # FIXME(jamielennox): update to use load_from_options_getter when
-        # https://review.openstack.org/162529 merges.
-
-        # !!! - UNDER NO CIRCUMSTANCES COPY ANY OF THIS CODE - !!!
+        # pattern we use in _conf_get. This function therefore does a manual
+        # version of load_from_conf_options with the fallback plugin inline.
 
         group = self._conf_get('auth_section') or _base.AUTHTOKEN_GROUP
 
@@ -1005,20 +1007,16 @@ class AuthProtocol(BaseAuthProtocol):
                                                  group=group)
             )
 
+        # Plugin option registration is normally done as part of the load_from
+        # function rather than the register function so copy here.
         plugin_loader = loading.get_plugin_loader(plugin_name)
-        plugin_opts = [o._to_oslo_opt() for o in plugin_loader.get_options()]
-        plugin_kwargs = dict()
+        plugin_opts = loading.get_auth_plugin_conf_options(plugin_loader)
 
         (self._local_oslo_config or CONF).register_opts(plugin_opts,
                                                         group=group)
 
-        for opt in plugin_opts:
-            val = self._conf_get(opt.dest, group=group)
-            if val is not None:
-                val = opt.type(val)
-            plugin_kwargs[opt.dest] = val
-
-        return plugin_loader.load_from_options(**plugin_kwargs)
+        getter = lambda opt: self._conf_get(opt.dest, group=group)
+        return plugin_loader.load_from_options_getter(getter)
 
     def _determine_project(self):
         """Determine a project name from all available config sources.
@@ -1118,7 +1116,7 @@ class AuthProtocol(BaseAuthProtocol):
 
 
 def filter_factory(global_conf, **local_conf):
-    """Returns a WSGI filter app for use with paste.deploy."""
+    """Return a WSGI filter app for use with paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
 
