@@ -15,8 +15,8 @@ import uuid
 
 import fixtures
 from keystoneauth1 import fixture as client_fixtures
-from keystoneclient import utils
 from oslo_utils import timeutils
+from positional import positional
 
 from keystonemiddleware import auth_token
 from keystonemiddleware.auth_token import _exceptions
@@ -43,7 +43,7 @@ class AuthTokenFixture(fixtures.Fixture):
     def tokens(self):
         return self._token_data.keys()
 
-    @utils.positional(1)
+    @positional(1)
     def add_token_data(self, token_id=None, expires=None,
                        user_id=None, user_name=None,
                        user_domain_id=None, user_domain_name=None,
@@ -51,7 +51,6 @@ class AuthTokenFixture(fixtures.Fixture):
                        project_domain_id=None, project_domain_name=None,
                        role_list=None, is_v2=False):
         """Add token data to the auth_token fixture."""
-
         if not token_id:
             token_id = uuid.uuid4().hex
 
