@@ -17,12 +17,11 @@ import uuid
 
 import mock
 from oslo_config import cfg
-from pycadf import identifier
-import testtools
 from testtools import matchers
 import webob
 
 from keystonemiddleware import audit
+from keystonemiddleware.tests.unit import utils
 
 
 class FakeApp(object):
@@ -40,7 +39,7 @@ class FakeFailingApp(object):
         raise Exception('It happens!')
 
 
-class BaseAuditMiddlewareTest(testtools.TestCase):
+class BaseAuditMiddlewareTest(utils.BaseTestCase):
     def setUp(self):
         super(BaseAuditMiddlewareTest, self).setUp()
         self.fd, self.audit_map = tempfile.mkstemp()
@@ -144,6 +143,7 @@ class AuditMiddlewareTest(BaseAuditMiddlewareTest):
     def test_process_request_fail(self):
         req = webob.Request.blank('/foo/bar',
                                   environ=self.get_environ_header('GET'))
+        req.context = {}
         with mock.patch('oslo_messaging.Notifier.info',
                         side_effect=Exception('error')) as notify:
             self.middleware._process_request(req)
@@ -152,6 +152,7 @@ class AuditMiddlewareTest(BaseAuditMiddlewareTest):
     def test_process_response_fail(self):
         req = webob.Request.blank('/foo/bar',
                                   environ=self.get_environ_header('GET'))
+        req.context = {}
         with mock.patch('oslo_messaging.Notifier.info',
                         side_effect=Exception('error')) as notify:
             self.middleware._process_response(req, webob.response.Response())
@@ -200,6 +201,25 @@ class AuditMiddlewareTest(BaseAuditMiddlewareTest):
                 self.assertEqual('audit.http.response',
                                  call_args[1]['event_type'])
 
+    def test_cadf_event_context_scoped(self):
+        middleware = audit.AuditMiddleware(
+            FakeApp(),
+            audit_map_file=self.audit_map,
+            service_name='pycadf')
+        req = webob.Request.blank('/foo/bar',
+                                  environ=self.get_environ_header('GET'))
+        with mock.patch('oslo_messaging.Notifier.info') as notify:
+            middleware(req)
+
+            self.assertEqual(2, notify.call_count)
+            first, second = [a[0] for a in notify.call_args_list]
+
+            # the Context is the first argument. Let's verify it.
+            self.assertIsInstance(first[0], dict)
+
+            # ensure exact same context is used between request and response
+            self.assertIs(first[0], second[0])
+
     def test_cadf_event_scoped_to_request(self):
         middleware = audit.AuditMiddleware(
             FakeApp(),
@@ -222,12 +242,14 @@ class AuditMiddlewareTest(BaseAuditMiddlewareTest):
             service_name='pycadf')
         req = webob.Request.blank('/foo/bar',
                                   environ=self.get_environ_header('GET'))
+        req.context = {}
         with mock.patch('oslo_messaging.Notifier.info',
                         side_effect=Exception('error')) as notify:
             middleware._process_request(req)
             self.assertTrue(notify.called)
         req2 = webob.Request.blank('/foo/bar',
                                    environ=self.get_environ_header('GET'))
+        req2.context = {}
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             middleware._process_response(req2, webob.response.Response())
             self.assertTrue(notify.called)
@@ -236,12 +258,111 @@ class AuditMiddlewareTest(BaseAuditMiddlewareTest):
                                 notify.call_args_list[0][0][2]['id'])
 
 
+def _get_transport(conf, aliases=None, url=None):
+    transport = mock.MagicMock()
+    transport.conf = conf
+    conf.register_opts = mock.MagicMock()
+    return transport
+
+
+@mock.patch('oslo_messaging.get_transport', side_effect=_get_transport)
+class AuditNotifierConfigTest(BaseAuditMiddlewareTest):
+
+    def test_conf_middleware_log_and_default_as_messaging(self, t):
+        cfg.CONF.notification_driver = ['messaging']  # MultiOptStr value
+        cfg.CONF.audit_middleware_notifications.driver = 'log'
+        middleware = audit.AuditMiddleware(
+            FakeApp(),
+            audit_map_file=self.audit_map,
+            service_name='pycadf')
+        req = webob.Request.blank('/foo/bar',
+                                  environ=self.get_environ_header('GET'))
+        req.context = {}
+        with mock.patch('oslo_messaging.notify._impl_log.LogDriver.notify',
+                        side_effect=Exception('error')) as driver:
+            middleware._process_request(req)
+            # audit middleware conf has 'log' make sure that driver is invoked
+            # and not the one specified in DEFAULT section
+            self.assertTrue(driver.called)
+
+    def test_conf_middleware_log_and_oslo_msg_as_messaging(self, t):
+        cfg.CONF.notification_driver = None
+        cfg.CONF.oslo_messaging_notifications.driver = ['messaging']
+        cfg.CONF.audit_middleware_notifications.driver = 'log'
+        middleware = audit.AuditMiddleware(
+            FakeApp(),
+            audit_map_file=self.audit_map,
+            service_name='pycadf')
+        req = webob.Request.blank('/foo/bar',
+                                  environ=self.get_environ_header('GET'))
+        req.context = {}
+        with mock.patch('oslo_messaging.notify._impl_log.LogDriver.notify',
+                        side_effect=Exception('error')) as driver:
+            middleware._process_request(req)
+            # audit middleware conf has 'log' make sure that driver is invoked
+            # and not the one specified in oslo_messaging_notifications section
+            self.assertTrue(driver.called)
+
+    def test_conf_middleware_messaging_and_oslo_msg_as_log(self, t):
+        cfg.CONF.notification_driver = None
+        cfg.CONF.oslo_messaging_notifications.driver = ['log']
+        cfg.CONF.audit_middleware_notifications.driver = 'messaging'
+        middleware = audit.AuditMiddleware(
+            FakeApp(),
+            audit_map_file=self.audit_map,
+            service_name='pycadf')
+        req = webob.Request.blank('/foo/bar',
+                                  environ=self.get_environ_header('GET'))
+        req.context = {}
+        with mock.patch('oslo_messaging.notify.messaging.MessagingDriver'
+                        '.notify',
+                        side_effect=Exception('error')) as driver:
+            # audit middleware has 'messaging' make sure that driver is invoked
+            # and not the one specified in oslo_messaging_notifications section
+            middleware._process_request(req)
+            self.assertTrue(driver.called)
+
+    def test_with_no_middleware_notification_conf(self, t):
+        cfg.CONF.notification_driver = None
+        cfg.CONF.oslo_messaging_notifications.driver = ['messaging']
+        cfg.CONF.audit_middleware_notifications.driver = None
+        middleware = audit.AuditMiddleware(
+            FakeApp(),
+            audit_map_file=self.audit_map,
+            service_name='pycadf')
+        req = webob.Request.blank('/foo/bar',
+                                  environ=self.get_environ_header('GET'))
+        req.context = {}
+        with mock.patch('oslo_messaging.notify.messaging.MessagingDriver'
+                        '.notify',
+                        side_effect=Exception('error')) as driver:
+            # audit middleware section is not set. So driver needs to be
+            # invoked from oslo_messaging_notifications section.
+            middleware._process_request(req)
+            self.assertTrue(driver.called)
+
+    def test_conf_middleware_messaging_and_transport_set(self, mock_transport):
+        transport_url = 'rabbit://me:passwd@host:5672/virtual_host'
+        cfg.CONF.audit_middleware_notifications.driver = 'messaging'
+        cfg.CONF.audit_middleware_notifications.transport_url = transport_url
+
+        audit.AuditMiddleware(
+            FakeApp(),
+            audit_map_file=self.audit_map,
+            service_name='pycadf')
+        self.assertTrue(mock_transport.called)
+        # make sure first call kwarg 'url' is same as provided transport_url
+        self.assertEqual(transport_url,
+                         mock_transport.call_args_list[0][1]['url'])
+
+
 @mock.patch('oslo_messaging.rpc', mock.MagicMock())
 class AuditApiLogicTest(BaseAuditMiddlewareTest):
 
     def api_request(self, method, url):
         req = webob.Request.blank(url, environ=self.get_environ_header(method),
                                   remote_addr='192.168.0.1')
+        req.context = {}
         self.middleware._process_request(req)
         return req
 
@@ -255,17 +376,17 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         self.assertEqual(payload['outcome'], 'pending')
         self.assertEqual(payload['eventType'], 'activity')
         self.assertEqual(payload['target']['name'], 'nova')
-        self.assertEqual(payload['target']['id'], 'openstack:resource_id')
+        self.assertEqual(payload['target']['id'], 'resource_id')
         self.assertEqual(payload['target']['typeURI'],
                          'service/compute/servers')
         self.assertEqual(len(payload['target']['addresses']), 3)
         self.assertEqual(payload['target']['addresses'][0]['name'], 'admin')
         self.assertEqual(payload['target']['addresses'][0]['url'],
                          'http://admin_host:8774')
-        self.assertEqual(payload['initiator']['id'], 'openstack:user_id')
+        self.assertEqual(payload['initiator']['id'], 'user_id')
         self.assertEqual(payload['initiator']['name'], 'user_name')
         self.assertEqual(payload['initiator']['project_id'],
-                         'openstack:tenant_id')
+                         'tenant_id')
         self.assertEqual(payload['initiator']['host']['address'],
                          '192.168.0.1')
         self.assertEqual(payload['initiator']['typeURI'],
@@ -318,7 +439,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         self.assertEqual(payload['action'], 'read/list')
         self.assertEqual(payload['outcome'], 'pending')
         self.assertEqual(payload['target']['name'], 'nova')
-        self.assertEqual(payload['target']['id'], 'openstack:resource_id')
+        self.assertEqual(payload['target']['id'], 'resource_id')
         self.assertEqual(payload['target']['typeURI'],
                          'service/compute/servers')
 
@@ -375,6 +496,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
                                   environ=self.get_environ_header('POST'))
         req.body = b'{"createImage" : {"name" : "new-image","metadata": ' \
                    b'{"ImageType": "Gold","ImageVersion": "2.0"}}}'
+        req.context = {}
         self.middleware._process_request(req)
         payload = req.environ['cadf_event'].as_dict()
         self.assertEqual(payload['target']['typeURI'],
@@ -418,6 +540,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
     def test_response_mod_msg(self):
         req = self.api_request('GET', 'http://admin_host:8774/v2/'
                                + str(uuid.uuid4()) + '/servers')
+        req.context = {}
         payload = req.environ['cadf_event'].as_dict()
         self.middleware._process_response(req, webob.Response())
         payload2 = req.environ['cadf_event'].as_dict()
@@ -434,6 +557,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
     def test_no_response(self):
         req = self.api_request('GET', 'http://admin_host:8774/v2/'
                                + str(uuid.uuid4()) + '/servers')
+        req.context = {}
         payload = req.environ['cadf_event'].as_dict()
         self.middleware._process_response(req, None)
         payload2 = req.environ['cadf_event'].as_dict()
@@ -450,6 +574,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         req = webob.Request.blank('http://admin_host:8774/v2/'
                                   + str(uuid.uuid4()) + '/servers',
                                   environ=self.get_environ_header('GET'))
+        req.context = {}
         self.assertNotIn('cadf_event', req.environ)
         self.middleware._process_response(req, webob.Response())
         self.assertIn('cadf_event', req.environ)
@@ -480,9 +605,10 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         req = webob.Request.blank('http://admin_host:8774/v2/'
                                   + str(uuid.uuid4()) + '/servers',
                                   environ=env_headers)
+        req.context = {}
         self.middleware._process_request(req)
         payload = req.environ['cadf_event'].as_dict()
-        self.assertEqual(payload['target']['id'], identifier.norm_ns('nova'))
+        self.assertEqual(payload['target']['id'], 'nova')
 
     def test_endpoint_missing_internal_url(self):
         env_headers = {'HTTP_X_SERVICE_CATALOG':
@@ -503,6 +629,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         req = webob.Request.blank('http://admin_host:8774/v2/'
                                   + str(uuid.uuid4()) + '/servers',
                                   environ=env_headers)
+        req.context = {}
         self.middleware._process_request(req)
         payload = req.environ['cadf_event'].as_dict()
         self.assertEqual((payload['target']['addresses'][1]['url']), "unknown")
@@ -526,6 +653,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         req = webob.Request.blank('http://admin_host:8774/v2/'
                                   + str(uuid.uuid4()) + '/servers',
                                   environ=env_headers)
+        req.context = {}
         self.middleware._process_request(req)
         payload = req.environ['cadf_event'].as_dict()
         self.assertEqual((payload['target']['addresses'][2]['url']), "unknown")
@@ -549,6 +677,7 @@ class AuditApiLogicTest(BaseAuditMiddlewareTest):
         req = webob.Request.blank('http://public_host:8774/v2/'
                                   + str(uuid.uuid4()) + '/servers',
                                   environ=env_headers)
+        req.context = {}
         self.middleware._process_request(req)
         payload = req.environ['cadf_event'].as_dict()
         self.assertEqual((payload['target']['addresses'][0]['url']), "unknown")
